"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createIndex = createIndex;
exports.createSyntheticIndex = createSyntheticIndex;
exports.createViewTrampoline = createViewTrampoline;
exports.getEntryPoint = getEntryPoint;
exports.indexFile = void 0;

var esbuild = _interopRequireWildcard(require("esbuild"));

var fs = _interopRequireWildcard(require("fs-extra"));

var parse5 = _interopRequireWildcard(require("parse5"));

var _escapeStringRegexp = _interopRequireDefault(require("escape-string-regexp"));

var _rewriteDependenciesPlugin = require("./plugins/rewriteDependenciesPlugin");

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var path = _interopRequireWildcard(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function createViewTrampoline(fileName, srcPath, dependencies, resolutions, browserTarget) {
  const fileRelativePath = `./${fileName}`;
  const trampolineTemplate = `
import ReactDOM from 'react-dom'
import React from 'react'
import Component from '${fileRelativePath}'
const DOMRoot = document.getElementById('root');
ReactDOM.render(<Component />, DOMRoot);`;
  const fileRegexp = new RegExp(String.raw`^${escapeRegex(fileRelativePath)}$`); // Build the trampoline on the fly, from stdin

  const buildResult = await esbuild.build({
    stdin: {
      contents: trampolineTemplate,
      resolveDir: srcPath,
      sourcefile: '_trampoline.tsx',
      loader: 'tsx'
    },
    format: 'esm',
    bundle: true,
    write: false,
    target: browserTarget,
    plugins: [// See https://github.com/evanw/esbuild/issues/456
    {
      name: 'import-path',

      setup(build) {
        build.onResolve({
          filter: fileRegexp
        }, args => {
          return {
            path: args.path,
            external: true
          };
        });
      }

    }, (0, _rewriteDependenciesPlugin.createRewriteDependenciesPlugin)({ ...dependencies,
      'react-dom': dependencies['react-dom'] ?? dependencies.react
    }, { ...resolutions,
      'react-dom': resolutions['react-dom'] ?? resolutions.react
    })]
  });
  return buildResult;
}

function escapeRegex(s) {
  return s.replace(/[-/\\^$*+?.()|[\]{}]/g, '\\$&');
}

function getEntryPoint(paths, metafile, type) {
  const result = Object.entries(metafile.outputs).find(([key, output]) => {
    return output.entryPoint && path.extname(key) === type;
  });
  const outputFileName = result === null || result === void 0 ? void 0 : result[0];

  if (outputFileName) {
    return path.relative(paths.appBuild, path.join((0, _getModularRoot.default)(), outputFileName));
  } else {
    return undefined;
  }
}

const indexFile = `
<!DOCTYPE html>
<html>
  <body>
    <div id="root"></div>
  </body>
</html>
`;
exports.indexFile = indexFile;

async function createIndex({
  paths,
  metafile,
  replacements,
  includeRuntime,
  indexContent,
  includeTrampoline
}) {
  const index = indexContent ?? (await fs.readFile(paths.appHtml, {
    encoding: 'utf-8'
  }));
  const cssEntryPoint = metafile ? getEntryPoint(paths, metafile, '.css') : undefined;
  const jsEntryPoint = metafile ? getEntryPoint(paths, metafile, '.js') : undefined;
  return compileIndex({
    indexContent: index,
    cssEntryPoint,
    jsEntryPoint,
    replacements,
    includeRuntime,
    includeTrampoline
  });
}

function createSyntheticIndex({
  cssEntryPoint,
  replacements
}) {
  return compileIndex({
    indexContent: indexFile,
    cssEntryPoint,
    replacements,
    includeTrampoline: true
  });
}

function compileIndex({
  indexContent,
  cssEntryPoint,
  jsEntryPoint,
  replacements,
  includeRuntime,
  includeTrampoline
}) {
  const page = parse5.parse(indexContent);
  const html = page.childNodes.find(node => node.nodeName === 'html');
  const head = html.childNodes.find(node => node.nodeName === 'head');
  const body = html.childNodes.find(node => node.nodeName === 'body');

  if (cssEntryPoint) {
    head.childNodes.push(...parse5.parseFragment(`<link rel="stylesheet" href="%PUBLIC_URL%/${cssEntryPoint}"></script>`).childNodes);
  }

  if (jsEntryPoint) {
    body.childNodes.push(...parse5.parseFragment(`<script type="module" src="%PUBLIC_URL%/${jsEntryPoint}"></script>`).childNodes);
  }

  if (includeTrampoline) {
    body.childNodes.push(...parse5.parseFragment(`<script type="module" src="%PUBLIC_URL%/static/js/_trampoline.js"></script>`).childNodes);
  }

  if (includeRuntime) {
    body.childNodes.push(...parse5.parseFragment(`<script type="module" src="%PUBLIC_URL%/_runtime/index.js"></script>`).childNodes);
  }

  let data = parse5.serialize(page); // Run HTML through a series of user-specified string replacements.

  Object.keys(replacements).forEach(key => {
    const value = replacements[key];
    data = data.replace(new RegExp('%' + (0, _escapeStringRegexp.default)(key) + '%', 'g'), value);
  });
  return data;
}
//# sourceMappingURL=api.js.map