"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cleanGit = cleanGit;
exports.getDiffedFiles = getDiffedFiles;
exports.stashChanges = stashChanges;

var _stripAnsi = _interopRequireDefault(require("strip-ansi"));

var _execa = _interopRequireDefault(require("execa"));

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function cleanGit(cwd) {
  const trackedChanged = (0, _stripAnsi.default)(_execa.default.sync('git', ['status', '-s'], {
    all: true,
    reject: false,
    cwd,
    cleanup: true
  }).stdout);
  return trackedChanged.length === 0;
}

function stashChanges() {
  _execa.default.sync('git', ['stash', '-u']);

  throw new Error('Failed to perform action cleanly. Stashing git changes...');
}

function getGitLocalDefaultBranch() {
  try {
    // this config may be set to a different default branch (e.g. main)
    const result = _execa.default.sync('git', ['config', 'init.defaultBranch']);

    return (0, _stripAnsi.default)(result.stdout);
  } catch (_err) {
    // if there init.defaultBranch hasn't been set, the default is master
    return 'master';
  }
}

function getGitDefaultBranch() {
  try {
    const result = _execa.default.sync('git', ['symbolic-ref', 'refs/remotes/origin/HEAD']);

    return `origin/${(0, _stripAnsi.default)(result.stdout).split('/').pop()}`;
  } catch (err) {
    // no remote origin, look into git config for init.defaultBranch setting
    return getGitLocalDefaultBranch();
  }
}

function getModifiedAndUntrackedFileChanges() {
  // get all modified and untracked files, excluding the standard Git exclusion sources
  const lsFiles = _execa.default.sync('git', ['ls-files', '-mo', '--exclude-standard'], {
    cwd: (0, _getModularRoot.default)()
  });

  if (lsFiles.stdout.length) {
    return (0, _stripAnsi.default)(lsFiles.stdout).split('\n');
  }

  return [];
} // Get all diffed files from git remote origin HEAD


function getGitDiff() {
  const defaultBranch = getGitDefaultBranch(); // get a commit sha between the HEAD of the current branch and git remote origin HEAD

  const sha = _execa.default.sync('git', ['merge-base', 'HEAD', defaultBranch], {
    cwd: (0, _getModularRoot.default)()
  });
  /**
   * diff-filter=ACMRTUB
   * (A) Added
   * (C) Copied
   * (M) Modified
   * (R) Renamed
   * (T) Type (i.e. regular file, symlink, submodule, …​) changed
   * (U) Unmerged
   * (B) Broken pairing (file that has had at least a certain percentage of its content deleted or changed)
   */


  const diff = _execa.default.sync('git', ['diff', '--name-only', '--diff-filter=ACMRTUB', sha.stdout], {
    cwd: (0, _getModularRoot.default)()
  });

  if (diff.stdout.length) {
    return (0, _stripAnsi.default)(diff.stdout).split('\n');
  }

  return [];
}

function getDiffedFiles() {
  return Array.from(new Set([...getGitDiff(), ...getModifiedAndUntrackedFileChanges()]).values());
}
//# sourceMappingURL=gitActions.js.map